# *****************************************************************
#
# Licensed Materials - Property of IBM 
#
# (C) Copyright IBM Corp. 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# ******************************************************************

from scipy import sparse
from _simsearch import libpywmd

#######################################
# COMPUTE WORD MOVERS DISTANCE
def word_movers_distance(W, X, Y=None, use_gpu=True):
    """Computes word movers distance between samples in X and Y.

    The details of the approximation algorithm can be found here:
    "http://export.arxiv.org/abs/1812.02091".

    Parameters
    ----------
    W : ndarray, shape: (n_features, n_dimensions)
        Embedding vectors

    X : array-like, sparse-matrix, shape (n_samples_x, n_features)
        Input dataset. When MPI is used, it is preferred to have X of larger size than Y when X and Y are of 
        unequal sizes as the MPI distribution of data is performed for X only.

    Y : array-like, sparse-matrix, shape (n_samples_y, n_features), optional
        Input dataset. If ``None``, the output will be the pairwise similarities between all samples in ``X``.

    Returns
    -------
    D : array-like, shape (n_samples_x, n_samples_y)
        Word Movers Distance
    """

    if Y is None:
        Y=X

    if (use_gpu==True):
        D = libpywmd.wmd_wrapper(W, X.nnz, X.indptr, X.indices, X.data, Y.nnz, Y.indptr, Y.indices, Y.data, 1)
    else:
        D = libpywmd.wmd_wrapper(W, X.nnz, X.indptr, X.indices, X.data, Y.nnz, Y.indptr, Y.indices, Y.data, 0)
    return(D)

#######################################
